#!/bin/bash
#
# Shell script to check the md5sum of the Objects
# "before" AND "after" a/some change(s), if there are any differences
#
# Copyright 2014 by guy maurel
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# 8 january 2014
#
#set -x
WHERE_PREPARE=/home/guy-kde/CodingStyle/
#
numberOfParameter=${#}
if [ ${numberOfParameter} -eq 0 ] ;
then
  echo "Use the script with one command"
  echo "  Command can be:"
  echo "    save:  save the md5sum before any change"
  echo "    test:  test all the saved md5sum again the new one"
  echo "    clean: remove all the saved md5sum and the save-directory"
  exit 0
fi
#
CMD=${1}
if [ ${CMD} == "save" ] ;
then
  command="1"
else
  if [ ${CMD} == "test" ] ;
  then
    command="2"
  else
    if [ ${CMD} == "clean" ] ;
    then
      command="3"
    else
      echo "command" ${CMD} "not known"
      exit 0
    fi
  fi
fi
#
if [ ${CMD} == "clean" ] ;
then
  rm -rf save
  echo "all the saved md5sum and the save-directory are removed"
  exit 1
fi
if [ ${CMD} == "save" ] ;
then
  # make the "save"-directory
  mkdir -p save
else
  if [[ ! -d save ]] ;
  then
    echo "the save-directory is not present"
    echo "run the script with the save-command before"
    echo "  Md5sum-the-Objects.sh save"
    exit 0
  fi
fi
#
if [ ${CMD} == "save" ] ;
then
  # make a list of the md5sum to be saved
  find . -type f -name "*.o" > save/to_be_saved.list
  LIST=`cat save/to_be_saved.list`
  for object_file in ${LIST}
  do
    echo "save the md5sum for "${object_file}
    # save the md5sum
    WHERE_TO_SAVE=`dirname ${object_file}`
    #echo "save/WHERE_TO_SAVE "save/${WHERE_TO_SAVE}
    mkdir -p save/${WHERE_TO_SAVE}
    md5sum ${object_file} > save/${object_file}.md5sum
    #ls -l ${object_file}
    cp ${object_file} save/${object_file}
    bName=`basename -s .o ${object_file}`
    #echo "bName of "${object_file}" is "${bName}
    if [ -f ${WHERE_TO_SAVE}/${bName}.s ] ;
    then 
      cp ${WHERE_TO_SAVE}/${bName}.s save/${WHERE_TO_SAVE}
    fi
  done
  touch save/time_stamp.txt
  echo "all md5sum are saved"
  exit 1
else
  #set -x
  # the command is "test" it
  # look for the new md5sum. They are newer as save/time_stamp.txt
  find . -type f -name "*.o" | grep -v "/save/" > save/new_md5sum.list
  LIST=`cat save/new_md5sum.list`
  typeset -i fileNumber=0
  for object_file in ${LIST}
  do
    fileNumber+=1
    echo ${fileNumber} "test the md5sum for "${object_file}
#exit
    md5sum --check --status save/${object_file}.md5sum > /dev/null 2>&1
    how_different=${?}
    if [ ${how_different} != "0" ] ;
    then
      echo "the status of md5sum --check --status is "${how_different}
      echo "for "${object_file}
  #  else
      WHERE_TO_SAVE=`dirname ${object_file}`
      bName=`basename -s .o ${object_file}`
      if [ -f ${WHERE_TO_SAVE}/${bName}.s ] ;
      then 
        echo "test the assembler code of "${object_file}
        # put the ".loc " lines away before compare
        gawk -f ${WHERE_PREPARE}/loc-lines-away.awk ${WHERE_TO_SAVE}/${bName}.s > a
        gawk -f ${WHERE_PREPARE}/loc-lines-away.awk save/${WHERE_TO_SAVE}/${bName}.s > b
        #diff ${WHERE_TO_SAVE}/${bName}.s save/${WHERE_TO_SAVE}/${bName}.s >/dev/null 2>&1
        diff a b >/dev/null 2>&1
        rm a
        rm b
        how_different=${?}
        if [ ${how_different} != "0" ] ;
        then
          echo "the status of diff is "${how_different}
          echo "for "${object_file}
          echo "diff ${WHERE_TO_SAVE}/${bName}.s save/${WHERE_TO_SAVE}/${bName}.s"
          ls -l ${WHERE_TO_SAVE}/${bName}.s save/${WHERE_TO_SAVE}/${bName}.s
          exit 0
        fi
      else
        echo "no assembler file for "${WHERE_TO_SAVE}/${bName}.s
      fi
    fi
  done
  rm save/new_md5sum.list
  echo "all tests are OK"
  exit 1
fi
